import time
import asyncio
from abc import ABC, abstractmethod # HELL

from asyncio.windows_events import ProactorEventLoop
from asyncio.events import AbstractEventLoop

class SelfPurgingObject(ABC): # You make this the parent, you like extend from it maybe??
    """
    The object will be auto purged from memory after a set amount of time, make sure the object has a __del__ function that handles saving if that is needed
    Requires the main function to use async
    """
    def __init__(self, time_till_purge_in_seconds: int, reset_timer_if_accessed: bool = True, event_loop: ProactorEventLoop | AbstractEventLoop = None):
        self.event_loop = event_loop
        self.purge_time_delay = time_till_purge_in_seconds
        self.reset_time_if_accessed = reset_timer_if_accessed
        self.time_to_be_deleted = None
        self.current_event_loop_call = None
        if self.event_loop is not None:
            self.after_pickle()
        else:
            print("You need to run .after_pickle()")


    def after_pickle(self):
        """Called after you pickle and save"""
        self.time_to_be_deleted = self.event_loop.time() + self.purge_time_delay
        self.schedule_deletion_for_purge_time()

    def before_pickle(self):
        """Called before you pickle and save"""
        self.event_loop = None
        self.time_to_be_deleted = None
        self.current_event_loop_call = None

    def reset_timer(self):
        if self.reset_time_if_accessed:
            self.time_to_be_deleted = self.event_loop.time() + self.purge_time_delay
            self.schedule_deletion_for_purge_time()
        else:
            pass # Don't reset timer duh

    def __delete_object(self):
        self._save() # SAVE
        del self # Make sure we are deleted

    def delete_if_time_finished_passing(self):
        if self.event_loop.time() >= self.time_to_be_deleted - 5: # 5 seconds just in case it calls it like a millisecond beforehand
            self.__delete_object()
        else:
            print("Not ready to be deleted")

    def schedule_deletion_for_purge_time(self):
        if self.current_event_loop_call is not None:
            try:
                self.current_event_loop_call.cancel()
            except():
                pass
        if self.time_to_be_deleted is not None:
            self.current_event_loop_call = self.event_loop.call_at(self.time_to_be_deleted, self.delete_if_time_finished_passing)
        else:
            pass

    @abstractmethod
    def _save(self):
        """Called by the on del function, must be overriden, if you wish not use it just have pass in the function"""
        pass

